import Vue from 'vue'
import throttle from '@utils/throttle.js'
const ctx = '@@InfiniteScroll'

var getScrollTop = function(element) {
  if (element === window) {
    return Math.max(window.pageYOffset || 0, document.documentElement.scrollTop)
  }

  return element.scrollTop
}

var getComputedStyle = Vue.prototype.$isServer ? {} : document.defaultView.getComputedStyle

var getScrollEventTarget = function(element) {
  var currentNode = element
  // bugfix, see http://w3help.org/zh-cn/causes/SD9013 and http://stackoverflow.com/questions/17016740/onscroll-function-is-not-working-for-chrome
  while (currentNode && currentNode.tagName !== 'HTML' && currentNode.tagName !== 'BODY' && currentNode.nodeType === 1) {
    var overflowY = getComputedStyle(currentNode).overflowY
    if (overflowY === 'scroll' || overflowY === 'auto') {
      return currentNode
    }
    currentNode = currentNode.parentNode
  }
  return window
}

var getVisibleHeight = function(element) {
  if (element === window) {
    return document.documentElement.clientHeight
  }

  return element.clientHeight
}

var getElementTop = function(element) {
  if (element === window) {
    return getScrollTop(window)
  }
  return element.getBoundingClientRect().top + getScrollTop(window)
}

var isAttached = function(element) {
  var currentNode = element.parentNode
  while (currentNode) {
    if (currentNode.tagName === 'HTML') {
      return true
    }
    if (currentNode.nodeType === 11) {
      return false
    }
    currentNode = currentNode.parentNode
  }
  return false
}

var doBind = function() {
  // 以下的 this 或 directive，指的是 el[ctx] 对象
  // doBind 只会调用一次
  if (this.binded) return; // eslint-disable-line
  this.binded = true

  var directive = this

  // 指令绑定的元素
  var element = directive.el

  // 获取滚动的元素
  directive.scrollEventTarget = getScrollEventTarget(element)
  // 使用函数节流方式调用滚动监听方法，从而减少调用频率，以下为 200ms 一次的频率
  directive.scrollListener = throttle(doCheck.bind(directive), 50)
  directive.scrollEventTarget.addEventListener('scroll', directive.scrollListener)

  // 是否禁用无限加载，即虽然进入 doCheck 方法，但是不会进行 doCheck 的处理
  var disabledExpr = element.getAttribute('infinite-scroll-disabled')
  var disabled = false

  // 滚动监听调用方法进行节流，onScroll 只要在组件内methods 存在，不需要进行任何配置
  if (directive.vm.onScroll) {
    directive.vm.onScroll = throttle(directive.vm.onScroll, 100)
  }

  if (disabledExpr) {
    // 监听 infinite-scroll-disabled 绑定的变量
    this.vm.$watch(disabledExpr, function(value) {
      directive.disabled = value
      if (!value && directive.immediateCheck) {
        doCheck.call(directive)
      }
    })
    disabled = Boolean(directive.vm[disabledExpr])
  }
  directive.disabled = disabled

  // 值有效性校验
  var distanceExpr = element.getAttribute('infinite-scroll-distance')
  var distance = 0
  if (distanceExpr) {
    distance = Number(directive.vm[distanceExpr] || distanceExpr)
    if (isNaN(distance)) {
      distance = 0
    }
  }
  directive.distance = distance

  var immediateCheckExpr = element.getAttribute('infinite-scroll-immediate-check')
  var immediateCheck = true
  if (immediateCheckExpr) {
    immediateCheck = Boolean(directive.vm[immediateCheckExpr])
  }
  directive.immediateCheck = immediateCheck

  if (immediateCheck) {
    doCheck.call(directive)
  }

  var eventName = element.getAttribute('infinite-scroll-listen-for-event')
  if (eventName) {
    directive.vm.$on(eventName, function() {
      doCheck.call(directive)
    })
  }
}

var doCheck = function(force) {
  // 滚动的元素
  var scrollEventTarget = this.scrollEventTarget

  // 指令绑定的元素，一般不等于scrollEventTarget
  var element = this.el

  // 触发无限加载事件的距离底部偏移
  var distance = this.distance

  if (force !== true && this.disabled) return; //eslint-disable-line

  // 当前滚动位置
  var viewportScrollTop = getScrollTop(scrollEventTarget)
  // 当前可视区域底部的位置
  var viewportBottom = viewportScrollTop + getVisibleHeight(scrollEventTarget)

  var shouldTrigger = false

  if (scrollEventTarget === element) {
    shouldTrigger = scrollEventTarget.scrollHeight - viewportBottom <= distance
  } else {
    // 滚动区域底部的位置
    var elementBottom = getElementTop(element) - getElementTop(scrollEventTarget) + element.offsetHeight + viewportScrollTop
    // 是否触发事件
    shouldTrigger = viewportBottom + distance >= elementBottom
  }

  // 滚动监听调用方法调用
  this.vm.onScroll && this.vm.onScroll(viewportScrollTop)

  if (shouldTrigger && this.expression) {
    this.expression()
  }
}

export default {
  bind(el, binding, vnode) {
    el[ctx] = {
      el,
      vm: vnode.context,
      expression: binding.value
    }
    const args = arguments
    var cb = function() {
      el[ctx].vm.$nextTick(function() {
        if (isAttached(el)) {
          doBind.call(el[ctx], args)
        }

        el[ctx].bindTryCount = 0

        var tryBind = function() {
          if (el[ctx].bindTryCount > 10) return; //eslint-disable-line
          el[ctx].bindTryCount++
          if (isAttached(el)) {
            doBind.call(el[ctx], args)
          } else {
            setTimeout(tryBind, 50)
          }
        }

        tryBind()
      })
    }
    if (el[ctx].vm._isMounted) {
      cb()
      return
    }
    el[ctx].vm.$on('hook:mounted', cb)
  },

  unbind(el) {
    if (el[ctx] && el[ctx].scrollEventTarget) {
      el[ctx].scrollEventTarget.removeEventListener('scroll', el[ctx].scrollListener)
    }
  }
}
